#!/usr/bin/perl -w
#
# extract llr.sh log file with multiple uuencoded files
# to HTML Report + Files
#
# usage: extract.pl <logfile>
#
# 2007,2008 Enno Ewers
#
# $Id: llr-extract.pl,v 1.26 2008/08/08 08:03:05 enno Exp $
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA  02110-1301, USA

sub usage()
{
    print STDERR "Usage: ./extract.pl <logfile>\n";
    print STDERR "   Extracts the contents of a linux-ir-log from <logfile>.\n";
    print STDERR "   An HTML version with embedded files linked will be\n";
    print STDERR "   written to <logfile>.html, the files will be written to\n";
    print STDERR "   the directory <logfile>.d\n";
}

sub print_tag
{
    if ($curtag ne "$_[0]") {
	print OUT "</$curtag>\n" if ($curtag);
	print OUT "<$_[0]>" if ($_[0]);
	$curtag = $_[0];
    }
}

sub print_code
{
    print_tag("code");
    print OUT "$_[0]";
}

sub print_plain
{
    print_tag(undef);
    print OUT "$_[0]";
}

%HTMLEscape = (
  '>' => '&gt;',
  '<' => '&lt;',
  '&' => '&amp;',
  \x8b => '&#139;',
  \x9b => '&#155;'
);

sub htmlescape
{
    my ($str) = @_;
    $str =~ s/([<>&\x8b\x9b])/$HTMLEscape{$1}/g;
    return $str;
}

sub uriescape
{
    my ($str) = @_;
    $str =~ s/([^A-Za-z0-9\-\/_.!~*’()])/sprintf("%%%02x",ord($1))/eg;
    return $str;
}

sub newsection
{
    my ($x) = @_;
    $section=$x;
    $section =~ s/[^a-zA-Z0-9_]//g;
    push @order, $section if (!exists $doc{$section});
    $doc{$section}{_title} = $x;
    newsubsec("_main");
}

sub newsubsec
{
    my ($x) = @_;
    $subsec=$x;
    $subsec =~ s/ /_/g;
    $subsec =~ s/[^a-zA-Z0-9_]//g;
    push @{$order{$section}{_section}}, $subsec if
	    (!exists $doc{$section}{$subsec});
    $doc{$section}{$subsec}{_title} = "$x";
    newcommand("_main");
}

sub newcommand
{
    my ($x) = @_;

    if ($x ne "_main") {
	$global_command_count++;
	$command = "cmd_$global_command_count";
    } else {
	$command = "_main";
    }
    push @{$order{$section}{$subsec}}, $command;
    $doc{$section}{$subsec}{$command}{_title} = "$x";
}

sub printstatus
{
    my ($percent,$msg) = (".");

    if ($logsize) {
	$percent = int($readcount/$logsize*100);
    }

    $msg = "\rParsing (${percent}%): $section/$subsec $_[0] ";

    print STDERR "\r" . (" " x ($lastl)) . $msg;

    $lastl = length($msg);
}

sub strings
{
    my ($file,$flink,$count) = @_;
    my $ret = "";
    my $i=0;

    open(SF,"$file");
    local $/ = undef;
    local $_ = <SF>;
    close(SF);
    printstatus("[file $flink.strings.html]");
    open(DS,">$file.strings.html");
    print DS "<html><head><title>Strings of file &quot;"
	.htmlescape($flink)
	."&quot;</title></head><body>"
	."<h1>Strings of file &quot;".htmlescape($flink)."</h1>"
	."<pre class=\"code\">";
    while (m/\G.*?([\x20-\x7e]{5,})/sg) {
	print DS htmlescape("$1\n");
        $ret .= "$1\n" if (++$i < $count);
    }
    print DS "\n</pre></body></html>\n";
    close(DS);

    return $ret;
}

#
# main
#
if (!@ARGV) {
    usage();
    exit(1);
}

$out = $ARGV[0].".html";
$outd = $ARGV[0].".d";
$outdrel = $outd;
$outdrel =~ s,.*/,,;
$logfile = $ARGV[0];

if (! -r $ARGV[0]) {
    print STDERR "Error: Cannot read '${ARGV[0]}'.\n\n";
    usage();
    exit(1);
}

$logsize = (stat($logfile))[7];
$lastl = 0;

if (-e $out) {
    print STDERR "Error: '$out' exists - will not overwrite.\n\n";
    usage();
    exit(1);
}

if (-e $outd) {
    print STDERR "Error: '$outd' exists - will not overwrite.\n\n";
    usage();
    exit(1);
}

mkdir($outd) || die "Cannot create '$outd': $!";

open(OUT,">$out") || die "Cannot write to '$out': $!";

my $flen = 0;
my ($H_code,$H_text) = (0,0);

newsection("_main");

while (<>) {
    $readcount += length($_);
    chomp;
    if ($in) {
	if (/^[\x21-\x60]/) {
	    $chunk = (unpack("u", $_)||"");
	    print DECODE $chunk;
	    $flen += length($chunk);
	} elsif (/^end/) {
	    close(DECODE);
	    chmod(oct($mode),$file);
	    $in=0;
	    my $strings = strings($file,$filelink,20);
	    $doc{$section}{$subsec}{$command}{_files} .=
		"<p><a href=\"".uriescape($filelink)."\">".htmlescape($fname).
		"</a> ($flen bytes)</p>".
		"<pre class=\"code\"><b>Strings:</b><br>".
		htmlescape($strings).
		"<a href=\"".uriescape("$filelink.strings.html")."\" class=\"code\">".
		"[...more...]".
		"</a><br></pre>";
	} else {
	    print STDERR "Warning: no end for ($file)\n";
	    print STDERR ">$_";
	    close(DECODE);
	    chmod(oct($mode),$file);
	    $in=0;
	    $doc{$section}{$subsec}{$command}{_files} .=
		"<p><a href=\"".uriescape($filelink)."\">".htmlescape($fname).
		"</a> ($flen bytes, decode problem: no end marker)</p>";
	    $doc{$section}{$subsec}{$command}{_code} .= "$_\n";
	}
    } elsif (/^begin (\d+) (\S+)/) {
	$fname = $2;
	$mode = $1;
	$flen = 0;

	$fname =~ s,^.*/,,; # strip path
	$file = $outd."/".$fname;
	$filelink = $outdrel."/".$fname;

	printstatus("[file $fname]");
	if (-e $file) {
	    print STDERR "Error: '$file' exists - will not overwrite\n";
	    exit(1);
	}
	open(DECODE,">$file") || die "Cannot write to '$file': $!";
	$in = 1;
    } elsif (/^===Section> (.*)$/) {
	newsection($1);
	printstatus("");
    } elsif (/^===Subsec> (.*)$/) {
	newsubsec($1);
	printstatus("");
    } elsif (/^===Info> (.*)$/) {
	$doc{$section}{$subsec}{$command}{_info} .= "$1\n";
    } elsif (/^===Cmd> (.*)$/) {
	newcommand($1);
    } elsif (/^---Status> (.*)$/) {
	chomp($doc{$section}{$subsec}{$command}{_code});
	$doc{$section}{$subsec}{$command}{_exit} = $1;
    } else {
	$doc{$section}{$subsec}{$command}{_code} .= "$_\n";
    }
}


if ($in) {
    print STDERR "Warning: End of file reached - incomplete last file\n";
    $doc{$section}{$subsec}{$command}{_files} .=
	"<p><a href=\"".uriescape($file)."\">".htmlescape($fname).
	"</a> ($flen bytes, decode problem: no end marker)</p>";
}

printstatus("");
print STDERR "...  Done\n";
print STDERR "Writing '$out'...";

$title = htmlescape($logfile);

print OUT <<"EOH";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN">
<html><head>
<meta http-equiv="content-type" content="text/html; charset=ISO-8859-1">
<title>Initial Response Log '$title' - Linux Live Response</title>
<style type="text/css">
    body { 
	font: normal 100% serif;
	color: black;
	background-color: white;
	min-width: 40em; 
	margin-left: 16em;
    }
    pre {
	white-space: pre-wrap;
    }
    td {
	font-size: small;
    }
    h1, h2, h3 {
	font-size: medium;
	background-color: white;
	border-bottom: 2px solid lightblue;
	border-left: none;
	border-right: none;
    }
    h1 {
	font-size: large;
    }
    h3 {
	background-color: #cdf8ff; 
	border-top: 2px solid lightblue;
    }
    #navigation { 
	position: fixed;
	top: 0px;
	left: 0px;
	width: 25em;
	font-size: 60%;
	height: 100%;
	background-color: white;
	border: 1px solid silver;
	margin: 0 0 0 0;
	overflow: scroll;
	overflow-x: hidden;
	background-color: lightgrey;
    }
    #navigation p {
	font-size: small;
    }
    #navigation a {
	color: black;
	text-decoration: none;
    }
    #navigation a:hover {
	background-color: white;
    }
    ul.sec_list {
	list-style: none inside;
	margin: 0 0 0 0;
	padding-left: 1em;
	border: thin solid blue;
	border-right: none;
	background-color: lightblue;
    }
    ul.sec_list_sub {
	list-style: none inside;
	margin: 0 0 0 0;
	padding-left: 1em;
	border: thin solid blue;
	border-right: none;
	background-color: lightgrey;
    }
    ul.sec_list_cmd {
	list-style: none inside;
	margin: 0 0 0 0;
	padding-left: 0em;
	border: 1px solid #BBBBBB;
	border-right: none;
	background-color: #BBBBBB;
    }
    ._fail a {
	font: normal 100% courier;
	background-color: orange;
    }
    ._success a {
	font: normal 100% courier;
	background-color: lightgreen;
    }
    ._psuccess a {
	font: normal 100% courier;
	background-color: lightyellow;
    }
    td._success {
	color: black;
	background-color: lightgreen;
	font-weight: normal;
    }
    td._fail {
	color: black;
	background-color: orange;
	font-weight: normal;
    }
    .code {
	border: 1px solid blue;
	background-color: lightblue;
	margin-top: 0px;
	font-size: medium;
	color: black;
	font-weight: normal;
    }
    .cmd_table {
	background-color: blue;
	color: white;
	font-weight: bold;
	margin-bottom: 0px;
	border: 1px solid blue;
	border-bottom: none;
    }
    .cmd_table td {
	border-bottom: 1px solid white;
	white-space: line-wrap;
    }
    .cmd_table a:link {
	color: white;
	font-weight: bold;
    }
    .cmd_table a:visited {
	color: white;
	font-weight: normal;
    }
    .cmd_table a:hover {
	color: blue;
	background-color: white;
    }
    .infoval {
	background-color: lightblue;
	color: black;
	font: normal 100% monospace;
    }
    .infokey {
	font: normal 100% monospace;
    }
    .cmdline {
	background-color: lightblue;
	font: normal 100% monospace;
	color: black;
    }
    .toggle {
        border: 1px solid black;
        background-color: white;
        color: black;
        margin: 1 1 1 1;
        font: normal 150% monospace;
    }
</style>
<script type="text/javascript">
function extend(obj,id) {
    var lis = obj.getElementsByTagName("ul");
    for (var i=0;i<lis.length;i++) {
	if (lis[i].className == id) {
	    lis[i].style.display = "block";
	}
    }
    lis = obj.getElementsByTagName("span");
    for (var i=0;i<lis.length;i++) {
	if (lis[i].className == "toggle") {
		lis[i].firstChild.nodeValue = "-";
	}
    }
}

function toggle(what) {
    var me = document.getElementById(what + "_toggle");
    var what = document.getElementById(what);
    if (me.firstChild.nodeValue == "-") {
	me.firstChild.nodeValue="+";
	collapse(what,"sec_list_cmd");
    } else {
	me.firstChild.nodeValue="-";
	extend(what,"sec_list_cmd");
    }
}

function extendall() {
	var nav = document.getElementById("navigation");
	extend(nav,"sec_list_cmd");
}
function collapse(obj,id) {
    var lis = obj.getElementsByTagName("ul");
    for (var i=0;i<lis.length;i++) {
	if (lis[i].className == id) {
		lis[i].style.display = "none";
	}
    }
    lis = obj.getElementsByTagName("span");
    for (var i=0;i<lis.length;i++) {
	if (lis[i].className == "toggle") {
		lis[i].firstChild.nodeValue = "+";
	}
    }
}
function collapseall() {
    var nav = document.getElementById("navigation");
    collapse(nav,"sec_list_cmd");
}
</script>
</head><body onload="toggle('id_Processes')">
<div id="navigation">
<a href=\"javascript:collapseall()\">[collapse]</a>
<a href=\"javascript:extendall()\">[extend]</a>
<ul class="sec_list">
<li><a href="#Main">Main</a></li>
EOH

for $section (@order) {
    print OUT "<li><a href=\"javascript:toggle('id_$section')\"><span class=\"toggle\" id=\"id_${section}_toggle\">-</span><a href=\"#sec_$section\">".
	htmlescape($doc{$section}{_title})."</li>\n"
	unless ($section eq "_main");
    print OUT "<ul class=\"sec_list_sub\" id=\"id_$section\">"
	if ($#{$order{$section}{_section}} > 0);
    for $subsec (@{$order{$section}{_section}}) {
	print OUT "<li><a href=\"#sub_$subsec\">".
	    htmlescape($doc{$section}{$subsec}{_title})."</li>\n"
	    unless ($subsec eq "_main");
	print OUT "<ul class=\"sec_list_cmd\">"
	    if ($#{$order{$section}{$subsec}} > 0);
	for $command (@{$order{$section}{$subsec}}) {
	    next if ($command eq "_main");
	    my $cref = $doc{$section}{$subsec}{$command};
	    my $div;
	    if ($cref->{_exit} eq "0") {
		$div = "_success";
	    } else {
		$div = "_fail";
	    }
	    my $cmdtitle = $cref->{_title};
	    if (length($cmdtitle) > 40) {
		$cmdtitle = substr($cmdtitle,0,40) . "[...]";
	    }
	    print OUT "<li class=\"$div\"><a href=\"#$command\">".
		htmlescape($cmdtitle)."</li>\n";
	}
	print OUT "</ul>\n" if ($#{$order{$section}{$subsec}} > 0);
    }
    print OUT "</ul>\n" if ($#{$order{$section}{_section}} > 0);
}
print OUT "</div>\n";
print OUT "<div id=\"Inhalt\"><a name=\"Main\">\n";
print OUT "<h1>Initial Response Log '$title' - Linux Live Reponse</h1>\n";

for $section (@order) {
    print OUT "<h2><a name=\"sec_$section\">".
	htmlescape($doc{$section}{_title})."</h2>\n"
	unless ($section eq "_main");
    for $subsec (@{$order{$section}{_section}}) {
	print OUT "<h3><a name=\"sub_$subsec\">".
	    htmlescape($doc{$section}{$subsec}{_title})."</h3>\n"
	    unless ($subsec eq "_main");
	for $command (@{$order{$section}{$subsec}}) {
	    print OUT "<div class=\"cmd\">";
	    print OUT "<table class=\"cmd_table\" border=\"0\" cellspacing=\"0\">";
	    if ($command ne "_main") {
		print OUT "<a name=\"${command}\">";
		print OUT "<tr><td>Command:</td><td class=\"cmdline\">".
		    htmlescape($doc{$section}{$subsec}{$command}{_title}).
		    "</td></tr>\n";
		if (exists $doc{$section}{$subsec}{$command}{_exit}) {
		    $ex = $doc{$section}{$subsec}{$command}{_exit};
		} else {
		    $ex = "Exit status unknown";
		}
		print OUT "<tr><td>Exit status:</td><td ".
		    (($ex eq "0") ?
			"class=\"_success\">Sucess (0)" :
			"class=\"_fail\">Failed ($ex)")
		    ."</td></tr>\n";
	    }
	    if (exists $doc{$section}{$subsec}{$command}{_info}) {
		@l = split("\n",
		    htmlescape($doc{$section}{$subsec}{$command}{_info}));
		for (@l) {
		    ($k,$v) = split(":",$_,2);
		    $v =~ s/^\s+//;
		    print OUT "<tr class=\"info\"><td class=\"infokey\">$k</td><td class=\"infoval\">$v</td></tr>";
		}
	    }
	    if (exists $doc{$section}{$subsec}{$command}{_files}) {
		print OUT "<tr><td>Files:</td><td>".
		    $doc{$section}{$subsec}{$command}{_files}."</td></tr>\n";
	    }
	    print OUT "</table>\n";
	    if (exists $doc{$section}{$subsec}{$command}{_code}) {
		print OUT "<pre class=\"code\">".
		    htmlescape($doc{$section}{$subsec}{$command}{_code}).
		    "\n</pre>\n";
	    }
	    print OUT "</div>";
	}
    }
}

print OUT "</div></body></html>\n";

close(OUT);

print STDERR "Done\n";

print STDERR "HTML Log is in '$out'.\n";
